package com.agagliardi.z.utils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import com.agagliardi.z.utils.strategy.Strategy;
import com.agagliardi.z.utils.strategy.WhenBellTools;


public abstract class ZObject<E> implements Serializable,Triggerable
{

    private transient byte[] storage = null;

    private Bootable<E> delegate;

    private static final long serialVersionUID = 1L;
    
    private transient Strategy strategy=null;

    public ZObject()
    {
        super();
    }

    public ZObject(Bootable<E> delegate)
    {
    	this();
        this.delegate = delegate;
    }

    public ZObject(E delegate)
    {
        this();
        this.delegate = new FakeBootable<E>(delegate);
    }

    @Override
    public synchronized void reborn()
    {
        if (storage != null)
        {
        	assert (delegate==null);
            System.err.println("Reborn " + storage.length);
            ObjectInputStream ois = null;            
            ByteArrayInputStream bais = null;
            DataInputStream dis= null;  
            GZIPInputStream gzis = null;
            try
            {
                bais = new ByteArrayInputStream(storage);
                gzis = new GZIPInputStream(bais);
                ois = new ObjectInputStream(gzis);
                delegate = (Bootable<E>) ois.readObject();
                ois.close();

                delegate.boot();
                storage = null;
            }
            catch (Exception e)
            {
                throw new RuntimeException(e);
            }
        }
    }

    @Override
    public synchronized void die()
    {
        if (storage == null)
        {
            System.err.println("Die");
            ByteArrayOutputStream baos =null;
            DataOutputStream dos = null;
            ObjectOutputStream oos = null;
            GZIPOutputStream gzous = null;
            try
            {
            	baos = new ByteArrayOutputStream();
				gzous = new GZIPOutputStream(baos);
                oos = new ObjectOutputStream(gzous);
                delegate.compact();
                oos.writeObject(delegate);
                oos.close();
                storage = baos.toByteArray();
                System.err.println("GZipped " + storage.length);
                delegate.defuse();
                delegate=null;
                strategy=null;
            }
            catch (IOException e)
            {
                e.printStackTrace();
                throw new RuntimeException(e);
            }
        }
    }

    protected E getAlive()
    {
    	if(strategy==null){
    		strategy = new WhenBellTools(this,1000);
    	}
    	strategy.karma();
        return delegate.getInner();
    }

    private void writeObject(ObjectOutputStream out) throws IOException
    {
        //die();
        reborn();
        out.defaultWriteObject();
    }

    private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException
    {
        in.defaultReadObject();
        //die();
        //this.trigger = new WeakReference(null);
    }

}
